package org.springblade.modules.system.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.ObjectUtils;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.StringUtil;
import org.springblade.modules.system.entity.PlatomixApp;
import org.springblade.modules.system.entity.PlatomixAppCopyLog;
import org.springblade.modules.system.entity.PlatomixAppGroup;
import org.springblade.modules.system.entity.PlatomixAppGroupServer;
import org.springblade.modules.system.rpc.CubeDataClientRpc;
import org.springblade.modules.system.service.*;
import org.springblade.system.client.model.UserModel;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static org.springblade.core.secure.utils.AuthUtil.getUser;

/**
 *
 * 应用管理相关接口
 *
 */
@RestController
@AllArgsConstructor
@Api(value = "应用查询", tags = "应用查询")
@RequestMapping("/app")
public class AppController extends BladeController {

    @Resource
    private PlatomixAppGroupService platomixAppGroupService;

    @Resource
    private PlatomixAppGroupServerService platomixAppGroupServerService;

    @Resource
    private PlatomixAppService platomixAppService;

    @Resource
    private PlatomixAppCopyLogService platomixAppCopyLogService;

    @Resource
    private CubeDataClientRpc cubeDataClientRpc;

    @Resource
    private PlatomixAppPermissionService appPermissionService;

//    @GetMapping("/list")
//    @ApiOperation(value = "应用列表", notes = "应用列表")
//    public R<List<PlatomixApp>> list() {
//        return R.data(platomixAppService.list());
//    }

    @ApiOperation(value = "获取应用列表(根据权限)", notes = "获取应用列表(根据权限)")
    @GetMapping("/list")
    public R<IPage<PlatomixApp>> page(
            @RequestParam(value = "current", required = false, defaultValue = "1") Integer current,
            @RequestParam(value = "size", required = false, defaultValue = "100") Integer size,
            @RequestParam(value = "name", required = false) String name) {
        LambdaQueryWrapper<PlatomixApp> wrapper = Wrappers.lambdaQuery();
        if (StrUtil.isNotBlank(name)) {
            wrapper.like(PlatomixApp::getAppName, name);
        }
        //只能查询到自己权限范围内表单信息
        appPermissionService.checkAppList(getUser().getUserId(), wrapper);
        Query query = new Query();
        query.setCurrent(current);
        query.setSize(size);
        IPage<PlatomixApp> pageData = platomixAppService.page(Condition.getPage(query), wrapper);
        return R.data(pageData);
    }


    @PostMapping("/add")
    @ApiOperation(value = "增加应用", notes = "增加应用")
    public R<PlatomixApp> add(@RequestBody PlatomixApp platomixApp) {
        platomixApp.setAppId(IdUtil.getSnowflakeNextIdStr());
        platomixApp.setCreateTime(new Date());
        platomixApp.setCreateId(AuthUtil.getUserId() + "");
        platomixApp.setTenantId(AuthUtil.getTenantId());
        //创建应用的数据源信息
        Long dataSourceId = cubeDataClientRpc.createReportDatasource(platomixApp.getAppId(), platomixApp.getAppName());
        if(ObjectUtils.isEmpty(dataSourceId)){
            return R.fail("创建应用数据源失败.");
        }
        platomixApp.setDataSourceId(dataSourceId);
        Boolean result =  platomixAppService.save(platomixApp);
        if(result){
            //自动创建“默认分组”目录
            PlatomixAppGroup platomixAppGroup = new PlatomixAppGroup();
            platomixAppGroup.setAppGroupId(IdUtil.getSnowflakeNextIdStr());
            platomixAppGroup.setAppGroupName("默认分组");
            platomixAppGroup.setAppId(platomixApp.getAppId());
            platomixAppGroup.setCreateTime(new Date());
            platomixAppGroup.setCreateId(AuthUtil.getUserId() + "");
            platomixAppGroup.setTenantId(AuthUtil.getTenantId());
            platomixAppGroup.setGroupOrder(1);
            platomixAppGroupService.save(platomixAppGroup);
            return R.data(platomixApp);
        }else{
            return R.fail("创建应用失败");
        }
    }

    @PostMapping("/updateById")
    @ApiOperation(value = "修改应用", notes = "修改应用")
    public R<PlatomixApp> c(@RequestBody PlatomixApp platomixApp) {
        platomixAppService.updateById(platomixApp);
        return R.data(platomixApp);
    }


    @GetMapping("/group/list/{appId}")
    @ApiOperationSupport(order = 3)
    @ApiOperation(value = "分类列表", notes = "分类列表")
    public R<List<PlatomixAppGroup>> groupList(@PathVariable("appId") String appId) {
        if (StringUtil.isBlank(appId)){
            return R.fail("应用id不能为空");
        }
        List<PlatomixAppGroup> list = platomixAppGroupService.lambdaQuery()
                .eq(PlatomixAppGroup::getAppId, appId)
                .orderByDesc(PlatomixAppGroup::getCreateTime).list();
        for (PlatomixAppGroup platomixAppGroup : list) {
            List<PlatomixAppGroupServer> list2 = platomixAppGroupServerService.lambdaQuery()
                    .eq(PlatomixAppGroupServer::getAppGroupId, platomixAppGroup.getAppGroupId())
                    .eq(PlatomixAppGroupServer::getAppId, platomixAppGroup.getAppId())
                    .orderByDesc(PlatomixAppGroupServer::getCreateTime).list();
            platomixAppGroup.setAppGroupServer(list2);
        }
        return R.data(list);
    }

    @PostMapping("/group/add")
    @ApiOperationSupport(order = 4)
    @ApiOperation(value = "分类新增", notes = "分类新增")
    public R<?> groupAdd( @RequestBody PlatomixAppGroup platomixAppGroup) {
        platomixAppGroup.setAppGroupId(IdUtil.getSnowflakeNextIdStr());
        platomixAppGroup.setCreateTime(new Date());
        platomixAppGroup.setCreateId(AuthUtil.getUserId() + "");
        platomixAppGroup.setTenantId(AuthUtil.getTenantId());
        platomixAppGroupService.save(platomixAppGroup);
        return R.data(platomixAppGroup);
    }

    @PostMapping("/group/updateById")
    @ApiOperation(value = "修改应用分类", notes = "修改应用分类")
    public R<PlatomixAppGroup> c(@RequestBody PlatomixAppGroup platomixApp) {
        platomixAppGroupService.updateById(platomixApp);
        return R.data(platomixApp);
    }

    @GetMapping("/group/server/list/{groupId}")
    @ApiOperationSupport(order = 5)
    @ApiOperation(value = " 分类下服务列表", notes = "分类下服务列表")
    public R submit(@PathVariable("groupId") String groupId) {
        if (StringUtil.isBlank(groupId)){
            return R.fail("应用id不能为空");
        }
        List<PlatomixAppGroupServer> list = platomixAppGroupServerService.lambdaQuery().eq(PlatomixAppGroupServer::getAppGroupId, groupId).orderByDesc(PlatomixAppGroupServer::getCreateTime).list();
        return R.data(list);
    }


    @PostMapping("/group/server/add/{groupId}")
    @ApiOperationSupport(order = 6)
    @ApiOperation(value = "新增关联", notes = "新增关联")
    public R remove( @RequestBody PlatomixAppGroupServer platomixAppGroup) {
        //platomixAppGroup.setAppGroupServerId(IdUtil.getSnowflakeNextIdStr());
        platomixAppGroup.setCreateTime(new Date());
        platomixAppGroup.setCreateId(AuthUtil.getUserId() + "");
        platomixAppGroup.setTenantId(AuthUtil.getTenantId());
        platomixAppGroupServerService.save(platomixAppGroup);
        return R.data(platomixAppGroup);
    }

    @GetMapping("/app/log/{appId}")
    @ApiOperationSupport(order = 6)
    @ApiOperation(value = "新增关联", notes = "新增关联")
    public R remove2(@PathVariable("appId") String appId) {
        List<PlatomixAppCopyLog> list = platomixAppCopyLogService.lambdaQuery().in(PlatomixAppCopyLog::getAppId, appId).orderByDesc(PlatomixAppCopyLog::getCreateTime).list();
        return R.data(list);
    }
}
