package org.springblade.modules.system.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.modules.system.dto.AppPermSaveForm;
import org.springblade.modules.system.entity.*;
import org.springblade.modules.system.service.*;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 *
 * 应用权限控制器
 *
 */
@RestController
@AllArgsConstructor
@Api(value = "应用权限控制器", tags = "应用权限控制器")
@RequestMapping("/app/permission")
public class AppPermissionController extends BladeController {

    @Resource
    private PlatomixAppService platomixAppService;

    @Resource
    private PlatomixAppPermissionService appPermissionService;

    @ApiOperation(value = "获取应用列表", notes = "获取应用列表")
    @GetMapping("/list")
    public R<IPage<PlatomixApp>> page(
            @RequestParam(value = "current", required = false, defaultValue = "1") Integer current,
            @RequestParam(value = "size", required = false, defaultValue = "100") Integer size,
            @RequestParam(value = "name", required = false) String name) {
        LambdaQueryWrapper<PlatomixApp> wrapper = Wrappers.lambdaQuery();
        if (StrUtil.isNotBlank(name)) {
            wrapper.like(PlatomixApp::getAppName, name);
        }
        Query query = new Query();
        query.setCurrent(current);
        query.setSize(size);
        IPage<PlatomixApp> pageData = platomixAppService.page(Condition.getPage(query), wrapper);
        return R.data(pageData);
    }


    /**
     * 根据权限组ID获取已关联表单信息
     * @param permId
     */
    @GetMapping("/list/{permId}")
    @ApiOperation(value = "根据权限组ID查询表单信息")
    public R<List<PlatomixAppPermission>> list(@PathVariable Long permId){
        if(ObjectUtil.isEmpty(permId)){
            return R.fail("permId不能为空值");
        }
        return R.data(appPermissionService.list(Wrappers.<PlatomixAppPermission>lambdaQuery().eq(PlatomixAppPermission::getDataPermId, permId)));
    }


    /**
     * 保存设置权限组表单信息
     * @param  permId
     * @param  saveForm
     * @return R<String>
     */
    @PostMapping("/save/{permId}")
    @ApiOperation(value = "设置权限组表单信息", notes = "设置权限组表单信息")
    @Transactional
    public R<String> doSave(@PathVariable Long permId, @RequestBody AppPermSaveForm saveForm){
        if(saveForm.getAddIds() == null || saveForm.getDelIds()==null){
            return R.fail("addIds或delIds不能为空");
        }
        if(saveForm.getAddIds()!=null && saveForm.getAddIds().size()>=1){
            List<PlatomixAppPermission> batchObjectList = new ArrayList<>();
            for(String appId : saveForm.getAddIds()){
                PlatomixAppPermission permission = new PlatomixAppPermission();
                permission.setAppId(appId);
                permission.setDataPermId(permId);
                permission.setCreateTime(new Date());
                permission.setCreateBy(getUser().getUserName());
                permission.setCreateUserId(getUser().getUserId()+"");
                permission.setCreateTime(new Date());
                permission.setTenantId(getUser().getTenantId());
                batchObjectList.add(permission);
            }
            if(CollectionUtil.isNotEmpty(batchObjectList)){
                appPermissionService.saveBatch(batchObjectList);
            }
        }
        if(saveForm.getDelIds()!=null && saveForm.getDelIds().size()>=1){
            appPermissionService.remove(Wrappers.<PlatomixAppPermission>lambdaQuery().in(PlatomixAppPermission::getAppId, saveForm.getDelIds()));
        }
        return R.success("保存成功.");
    }
}
