package org.springblade.modules.system.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springblade.modules.system.entity.DataPermission;
import org.springblade.modules.system.entity.PlatomixApp;
import org.springblade.modules.system.entity.PlatomixAppPermission;
import org.springblade.modules.system.mapper.PlatomixAppPermissionMapper;
import org.springblade.modules.system.service.PlatomixAppPermissionService;
import org.springblade.modules.system.service.normal.DataPermissionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Slf4j
@Service
public class PlatomixAppPermissionImpl extends ServiceImpl<PlatomixAppPermissionMapper, PlatomixAppPermission>
implements PlatomixAppPermissionService {


    @Resource
    private DataPermissionService dataPermissionServer;

    /**
     * 应用列表查询进行权限校验
     * @param  userId
     * @param  queryWrapper
     * @return void
     */
    @Override
    public void checkAppList(Long userId, LambdaQueryWrapper<PlatomixApp> queryWrapper){
        //查询用户所属权限
        List<DataPermission> datePermList = dataPermissionServer.getBaseMapper().selectOwnerList(userId);
        Boolean isSuperStatus = false;
        List<Integer> datePermIdList = new ArrayList<>();
        if(CollectionUtil.isNotEmpty(datePermList)){
            datePermIdList = datePermList.stream().map(DataPermission::getId).collect(Collectors.toList());
            for(DataPermission dataPermission : datePermList){
                if(dataPermission.getSuperStatus()==1){
                    isSuperStatus = true;
                    break;
                }
            }
        }
        //非管理员拼接SQL语句
        if(!isSuperStatus){
            if(CollectionUtil.isNotEmpty(datePermIdList)){
                String dataPermIdStr = String.join(",", datePermIdList.stream().map(Objects::toString).collect(Collectors.toList()));
                queryWrapper.and(wrapper->{
                    wrapper.eq(PlatomixApp::getCreateId, userId);
                    wrapper.or().inSql(PlatomixApp::getAppId, "select app_id from platomix_app_permission where data_perm_id IN("+dataPermIdStr+")");
                });
            }else{
                queryWrapper.eq(PlatomixApp::getCreateId,userId);
            }
        }
    }
}




