package org.springblade.modules.system.service.impl;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.modules.system.entity.PlatomixApp;
import org.springblade.modules.system.entity.PlatomixAppGroup;
import org.springblade.modules.system.entity.PlatomixAppGroupServer;
import org.springblade.modules.system.mapper.PlatomixAppMapper;
import org.springblade.modules.system.rpc.CubeDataClientRpc;
import org.springblade.modules.system.service.PlatomixAppGroupServerService;
import org.springblade.modules.system.service.PlatomixAppGroupService;
import org.springblade.modules.system.service.PlatomixAppService;
import org.springblade.system.client.model.PlatomixAppCopyModel;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 *
 */
@Slf4j
@Service
public class PlatomixAppServiceImpl extends ServiceImpl<PlatomixAppMapper, PlatomixApp>
implements PlatomixAppService {

    @Resource
    private PlatomixAppGroupServerService platomixAppGroupServerService;

    @Resource
    private PlatomixAppGroupService platomixAppGroupService;

    @Resource
    private CubeDataClientRpc cubeDataClientRpc;

    @Override
    @Transactional
    public Boolean delByAppId(Long appId) {
        PlatomixApp appEntity =  this.getOne(new QueryWrapper<PlatomixApp>().lambda().eq(PlatomixApp::getAppId, appId).eq(PlatomixApp::getTenantId, AuthUtil.getTenantId()));
        if(ObjectUtil.isEmpty(appEntity)){
            log.error("删除应用信息不存在，appId:{}", appId);
            return false;
        }
        Boolean result = this.removeById(appEntity.getId());
        if(result){
            platomixAppGroupServerService.remove(new QueryWrapper<PlatomixAppGroupServer>().lambda().eq(PlatomixAppGroupServer::getAppId, appId));
            platomixAppGroupService.remove(new QueryWrapper<PlatomixAppGroup>().lambda().eq(PlatomixAppGroup::getAppId, appId));
            if(ObjectUtil.isNotEmpty(appEntity.getDataSourceId())){
                //可以删除相应数据源信息，并清理数据源连接池
                cubeDataClientRpc.delReportDatasource(appEntity.getDataSourceId());
            }
        }
        return result;
    }


    @Override
    public PlatomixAppCopyModel copyApp(PlatomixAppCopyModel copyModel){
        PlatomixApp platomixApp = this.getOne(new QueryWrapper<PlatomixApp>().lambda().eq(PlatomixApp::getAppId, copyModel.getAppId()));
        if(ObjectUtil.isEmpty(platomixApp)){
            log.error("copyApp 应用信息不存在.");
            return null;
        }
        List<PlatomixAppGroup> appGroupList = platomixAppGroupService.list(new QueryWrapper<PlatomixAppGroup>().lambda().eq(PlatomixAppGroup::getAppId, copyModel.getAppId()));
        List<PlatomixAppGroupServer> groupServerList = platomixAppGroupServerService.list(new QueryWrapper<PlatomixAppGroupServer>().lambda().eq(PlatomixAppGroupServer::getAppId, copyModel.getAppId()));

        String nAppId = IdUtil.getSnowflakeNextIdStr();
        platomixApp.setId(null);
        platomixApp.setAppId(nAppId);
        platomixApp.setAppName(copyModel.getNewAppName());
        platomixApp.setDataSourceId(null);
        platomixApp.setCreateTime(new Date());
        platomixApp.setCreateId(AuthUtil.getUserId() + "");
        platomixApp.setTenantId(AuthUtil.getTenantId());
        Boolean result = this.save(platomixApp);
        if(result){
            for(PlatomixAppGroup appGroup : appGroupList){
                appGroup.setId(null);
                appGroup.setAppId(nAppId);
                appGroup.setCreateId(AuthUtil.getUserId() + "");
                appGroup.setCreateTime(new Date());
                appGroup.setUpdateId(AuthUtil.getUserId() + "");
                appGroup.setUpdateTime(new Date());
                appGroup.setTenantId(AuthUtil.getTenantId());
                platomixAppGroupService.save(appGroup);
            }
            for(PlatomixAppGroupServer entity : groupServerList){
                entity.setId(null);
                entity.setAppId(nAppId);
                entity.setCreateTime(new Date());
                entity.setCreateId(AuthUtil.getUserId() + "");
                entity.setTenantId(AuthUtil.getTenantId());
                platomixAppGroupServerService.save(entity);
            }
            copyModel.setNewAppId(Long.valueOf(nAppId));
            return copyModel;
        }
        log.error("复制应用失败，copyModel:{}", copyModel.toString());
        return copyModel;
    }
}




