/**
 *
 */
package org.springblade.resource;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.exceptions.ExceptionUtil;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.modules.system.entity.PlatomixApp;
import org.springblade.modules.system.entity.PlatomixAppCopyLog;
import org.springblade.modules.system.entity.PlatomixAppGroupServer;
import org.springblade.modules.system.service.PlatomixAppCopyLogService;
import org.springblade.modules.system.service.PlatomixAppGroupServerService;
import org.springblade.modules.system.service.PlatomixAppService;
import org.springblade.system.client.AppClient;
import org.springblade.system.client.model.PlatomixAppCopyLogModel;
import org.springblade.system.client.model.PlatomixAppCopyModel;
import org.springblade.system.client.model.PlatomixAppGroupServerModel;
import org.springblade.system.client.model.PlatomixAppModel;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @author wookvn
 *
 */
@Slf4j
@RestController
@ApiIgnore
public class AppResource implements AppClient {

    @Resource
    private PlatomixAppGroupServerService platomixAppGroupServerService;

    @Resource
    private PlatomixAppCopyLogService platomixAppCopyLogService;

    @Resource
    private PlatomixAppService platomixAppService;


    @Override
    public Boolean deleteByLinkId(String id, String type, String appId) {
        log.info("deleteByLinkId id:{}, type:{}, appId:{}", id, type, appId);
        try {
            List<PlatomixAppGroupServer> list = platomixAppGroupServerService.lambdaQuery()
                    .eq(PlatomixAppGroupServer::getLinkId, id)
                    .eq(PlatomixAppGroupServer::getAppId, appId)
                    .eq(PlatomixAppGroupServer::getLinkType, type).list();
            log.info("list:"+list.toString());
            for (PlatomixAppGroupServer platomixAppGroupServer : list) {
                platomixAppGroupServerService.removeById(platomixAppGroupServer);
            }
            return true;
        } catch (Exception e) {
            log.info(ExceptionUtil.stacktraceToString(e));
            return false;
        }
    }

    @Override
    public PlatomixAppCopyLogModel taskInfo(PlatomixAppCopyLogModel model) {
        PlatomixAppCopyLog platomixAppCopyLog = BeanUtil.copyProperties(model, PlatomixAppCopyLog.class);
        if (platomixAppCopyLog.getId()==null){
            platomixAppCopyLog.setCreateId(AuthUtil.getUserId()+"");
            platomixAppCopyLog.setCreateTime(new Date());
        }
        platomixAppCopyLogService.saveOrUpdate(platomixAppCopyLog);
        return   BeanUtil.copyProperties(platomixAppCopyLog, PlatomixAppCopyLogModel.class);
    }

    @Override
    public PlatomixAppGroupServerModel bind(PlatomixAppGroupServerModel model) {
        PlatomixAppGroupServer platomixAppGroupServer = BeanUtil.copyProperties(model, PlatomixAppGroupServer.class);
        Boolean result = platomixAppGroupServerService.save(platomixAppGroupServer);
        if(!result){
            return null;
        }
        return BeanUtil.copyProperties(platomixAppGroupServer, PlatomixAppGroupServerModel.class);
    }


    @Override
    public PlatomixAppModel getAppInfo(String appId) {
        if(StringUtils.isBlank(appId)){
            return null;
        }
        PlatomixApp entity = platomixAppService.lambdaQuery().eq(PlatomixApp::getAppId, appId).one();
        if(ObjectUtils.isEmpty(entity)){
            return null;
        }
        return BeanUtil.copyProperties(entity, PlatomixAppModel.class);
    }

    @Override
    public Boolean updateAppDatasoruceId(String appId, Long datasourceId) {
        return platomixAppService.update(null,
                new UpdateWrapper<PlatomixApp>().lambda()
                        .set(PlatomixApp::getDataSourceId, datasourceId)
                        .eq(PlatomixApp::getAppId, appId));
    }

    @Override
    public Boolean deleteByAppId(String appId) {
        return platomixAppService.delByAppId(Long.valueOf(appId));
    }

    @Override
    public PlatomixAppCopyModel copyProject(PlatomixAppCopyModel copyModel) {
       return  platomixAppService.copyApp(copyModel);
    }
}
