/**
 * 
 */
package org.springblade.system.client;

import org.apache.commons.collections4.BoundedCollection;
import org.springblade.common.openfeign.OpenFeignConstants;
import org.springblade.system.client.factory.DataPermClientFallbackFactory;
import org.springblade.system.client.model.PlatomixAppCopyLogModel;
import org.springblade.system.client.model.PlatomixAppCopyModel;
import org.springblade.system.client.model.PlatomixAppGroupServerModel;
import org.springblade.system.client.model.PlatomixAppModel;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

/**
 * @author wookvn
 *
 *	权限组
 */
@FeignClient(contextId = "appClient",
	value = OpenFeignConstants.SYSTEM_SERVICE, 
	fallbackFactory = DataPermClientFallbackFactory.class)
public interface AppClient {

	/**
	 * 根据likeId删除表单绑定关系
	 * @param id     表单ID
	 * @param type   类型
	 * @param appId  应用ID
	 * @return Boolean
	 */
	@GetMapping(OpenFeignConstants.SYSTEM_SERVICE + "/app/info/{id}/{type}/{appId}")
	public Boolean deleteByLinkId(@PathVariable String id, @PathVariable String type,@PathVariable String appId);

	/**
	 * 新增复制任务
	 * @param  model
	 * @return PlatomixAppCopyLogModel
	 */
	@PostMapping(OpenFeignConstants.SYSTEM_SERVICE + "/app/copy")
	public PlatomixAppCopyLogModel taskInfo(@RequestBody PlatomixAppCopyLogModel model);

	/**
	 * bind到应用分类
	 * @param  model
	 * @return PlatomixAppGroupServerModel
	 */
	@PostMapping(OpenFeignConstants.SYSTEM_SERVICE + "/app/bind")
	public PlatomixAppGroupServerModel bind(@RequestBody PlatomixAppGroupServerModel model);


	/**
	 * 获取应用基本信息
	 * @param  appId
	 * @return PlatomixAppModel
	 */
	@PostMapping(OpenFeignConstants.SYSTEM_SERVICE + "/app/info/{appId}")
	public PlatomixAppModel getAppInfo(@PathVariable String appId);


	/**
	 * 获取应用数据源ID值
	 * @param  appId
	 * @param  datasourceId
	 * @return Boolean
	 */
	@PostMapping(OpenFeignConstants.SYSTEM_SERVICE + "/app/update/{appId}/{datasourceId}")
	public Boolean updateAppDatasoruceId(@PathVariable String appId, @PathVariable Long datasourceId);

	/**
	 * 删除应用信息
	 * @param  appId
	 * @return PlatomixAppModel
	 */
	@PostMapping(OpenFeignConstants.SYSTEM_SERVICE + "/app/del/{appId}")
	public Boolean deleteByAppId(@PathVariable String appId);


	/**
	 * 复制应用信息
	 * @param  copyModel
	 * @return PlatomixAppCopyModel
	 */
	@PostMapping(OpenFeignConstants.SYSTEM_SERVICE + "/app/copy/project")
	public PlatomixAppCopyModel copyProject(@RequestBody PlatomixAppCopyModel copyModel);

}
